/**
 *  naken_asm assembler.
 *  Author: Michael Kohn
 *   Email: mike@mikekohn.net
 *     Web: http://www.mikekohn.net/
 * License: GPLv3
 *
 * Copyright 2010-2019 by Michael Kohn
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include "table/arc.h"

struct _table_arc table_arc[] =
{
  { "abs",    0x266f7009, 0xffff703f, OP_0_U6,  F_F },
  { "abs",    0x262f7f89, 0xffff7fff, OP_0_LIMM,F_F },
  { "abs",    0x262f7009, 0xffff703f, OP_0_C,   F_F },
  { "abs",    0x206f0009, 0xf8ff003f, OP_B_U6,  F_F },
  { "abs",    0x202f0f89, 0xf8ff0fff, OP_B_LIMM, F_F },
  { "abs",    0x202f0009, 0xf8ff003f, OP_B_C,    F_F },
  { "abss",   0x2e6f7005, 0xffff703f, OP_0_U6,   F_F },
  { "abss",   0x2e2f7f85, 0xffff7fff, OP_0_LIMM, F_F },
  { "abss",   0x2e2f7005, 0xffff703f, OP_0_C,    F_F },
  { "abss",   0x286f0005, 0xf8ff003f, OP_B_U6,   F_F },
  { "abss",   0x282f0f85, 0xf8ff0fff, OP_B_LIMM, F_F },
  { "abss",   0x282f0005, 0xf8ff003f, OP_B_C,    F_F },
  { "abssw",  0x2e6f7004, 0xffff703f, OP_0_U6,   F_F },
  { "abssw",  0x2e2f7f84, 0xffff7fff, OP_0_LIMM, F_F },
  { "abssw",  0x2e2f7004, 0xffff703f, OP_0_C,    F_F },
  { "abssw",  0x286f0004, 0xf8ff003f, OP_B_U6,   F_F },
  { "abssw",  0x2e2f7f84, 0xffff7fff, OP_B_LIMM, F_F },
  { "abssw",  0x282f0004, 0xf8ff003f, OP_B_C,    F_F },
  { "adc",    0x2041003e, 0xf8ff003f, OP_0_B_U6,    F_F },
  { "adc",    0x20010fbe, 0xf8ff0fff, OP_0_B_LIMM,  F_F },
  { "adc",    0x26c17000, 0xffff7020, OP_0_LIMM_C,  F_F | F_CC },
  { "adc",    0x2001003e, 0xf8ff003f, OP_0_B_C,     F_F },
  { "adc",    0x20410000, 0xf8ff0000, OP_A_B_U6,    F_F },
  { "adc",    0x20810000, 0xf8ff0000, OP_B_B_S12,   F_F },
  { "adc",    0x20c10020, 0xf8ff0020, OP_B_B_U6,    F_F | F_CC },
  { "adc",    0x26017000, 0xffff7000, OP_A_LIMM_C,  F_F },
  { "adc",    0x20010f80, 0xf8ff0fc0, OP_A_B_LIMM,  F_F },
  { "adc",    0x20c10f80, 0xf8ff0fe0, OP_B_B_LIMM,  F_F | F_CC },
  { "adc",    0x20c10000, 0xf8ff0020, OP_B_B_C,     F_F | F_CC },
  { "adc",    0x20010000, 0xf8ff0000, OP_A_B_C,     F_F },
  { "add",    0x2040003e, 0xf8ff003f, OP_0_B_U6,    F_F },
  { "add",    0x20000fbe, 0xf8ff0fff, OP_0_B_LIMM,  F_F },
  { "add",    0x26c07000, 0xffff7020, OP_0_LIMM_C,  F_F | F_CC },
  { "add",    0x2000003e, 0xf8ff003f, OP_0_B_C,     F_F },
  { "add",    0x20400000, 0xf8ff0000, OP_A_B_U6,    F_F },
  { "add",    0x20800000, 0xf8ff0000, OP_B_B_S12,   F_F },
  { "add",    0x20c00020, 0xf8ff0020, OP_B_B_U6,    F_F | F_CC },
  { "add",    0x26007000, 0xffff7000, OP_A_LIMM_C,  F_F },
  { "add",    0x20000f80, 0xf8ff0fc0, OP_A_B_LIMM,  F_F },
  { "add",    0x20c00f80, 0xf8ff0fe0, OP_B_B_LIMM,  F_F | F_CC },
  { "add",    0x20c00000, 0xf8ff0020, OP_B_B_C,     F_F | F_CC },
  { "add",    0x20000000, 0xf8ff0000, OP_A_B_C,     F_F },
  { "add1",   0x20140000, 0xf8ff0000, OP_A_B_C, F_F },
  { "add1",   0x20540000, 0xf8ff0000, OP_A_B_U6, F_F },
  { "add1",   0x20940000, 0xf8ff0000, OP_B_B_S12, F_F },
  { "add1",   0x20d40000, 0xf8ff0020, OP_B_B_C, F_F | F_CC },
  { "add1",   0x20d40020, 0xf8ff0020, OP_B_B_U6, F_F | F_CC },
  { "add1",   0x26147000, 0xffff7000, OP_A_LIMM_C, F_F },
  { "add1",   0x20140f80, 0xf8ff0fc0, OP_A_B_LIMM, F_F },
  { "add1",   0x20d40f80, 0xf8ff0fe0, OP_B_B_LIMM, F_F | F_CC },
  { "add1",   0x2014003e, 0xf8ff003f, OP_0_B_C, F_F },
  { "add1",   0x2054003e, 0xf8ff003f, OP_0_B_U6, F_F },
  { "add1",   0x20140fbe, 0xf8ff0fff, OP_0_B_LIMM, F_F },
  { "add1",   0x26d47000, 0xffff7020, OP_0_LIMM_C, F_F | F_CC },
  { "add2",   0x20150000, 0xf8ff0000, OP_A_B_C, F_F },
  { "add2",   0x20550000, 0xf8ff0000, OP_A_B_U6, F_F },
  { "add2",   0x20950000, 0xf8ff0000, OP_B_B_S12, F_F },
  { "add2",   0x20d50000, 0xf8ff0020, OP_B_B_C, F_F | F_CC },
  { "add2",   0x20d50020, 0xf8ff0020, OP_B_B_U6, F_F | F_CC },
  { "add2",   0x26157000, 0xffff7000, OP_A_LIMM_C, F_F },
  { "add2",   0x20150f80, 0xf8ff0fc0, OP_A_B_LIMM, F_F },
  { "add2",   0x20d50f80, 0xf8ff0fe0, OP_B_B_LIMM, F_F | F_CC },
  { "add2",   0x2015003e, 0xf8ff003f, OP_0_B_C, F_F },
  { "add2",   0x2055003e, 0xf8ff003f, OP_0_B_U6, F_F },
  { "add2",   0x20150fbe, 0xf8ff0fff, OP_0_B_LIMM, F_F },
  { "add2",   0x26d57000, 0xffff7020, OP_0_LIMM_C, F_F | F_CC },
  { "add3",   0x20160000, 0xf8ff0000, OP_A_B_C, F_F },
  { "add3",   0x20560000, 0xf8ff0000, OP_A_B_U6, F_F },
  { "add3",   0x20960000, 0xf8ff0000, OP_B_B_S12, F_F },
  { "add3",   0x20d60000, 0xf8ff0020, OP_B_B_C, F_F | F_CC },
  { "add3",   0x20d60020, 0xf8ff0020, OP_B_B_U6, F_F | F_CC },
  { "add3",   0x26167000, 0xffff7000, OP_A_LIMM_C, F_F },
  { "add3",   0x20160f80, 0xf8ff0fc0, OP_A_B_LIMM, F_F },
  { "add3",   0x20d60f80, 0xf8ff0fe0, OP_B_B_LIMM, F_F | F_CC },
  { "add3",   0x2016003e, 0xf8ff003f, OP_0_B_C, F_F },
  { "add3",   0x2056003e, 0xf8ff003f, OP_0_B_U6, F_F },
  { "add3",   0x20160fbe, 0xf8ff0fff, OP_0_B_LIMM, F_F },
  { "add3",   0x26d67000, 0xffff7020, OP_0_LIMM_C, F_F | F_CC },
  { "adds",   0x28060000, 0xf8ff0000, OP_A_B_C, F_F },
  { "adds",   0x28460000, 0xf8ff0000, OP_A_B_U6, F_F },
  { "adds",   0x28860000, 0xf8ff0000, OP_B_B_S12, F_F },
  { "adds",   0x28c60000, 0xf8ff0020, OP_B_B_C, F_F | F_CC },
  { "adds",   0x28c60020, 0xf8ff0020, OP_B_B_U6, F_F | F_CC },
  { "adds",   0x2e067000, 0xffff7000, OP_A_LIMM_C, F_F },
  { "adds",   0x28060f80, 0xf8ff0fc0, OP_A_B_LIMM, F_F },
  { "adds",   0x28c60f80, 0xf8ff0fc0, OP_B_B_LIMM, F_F | F_CC },
  { "adds",   0x2806003e, 0xf8ff003f, OP_0_B_C, F_F },
  { "adds",   0x2846003e, 0xf8ff003f, OP_0_B_U6, F_F },
  { "adds",   0x28060fbe, 0xf8ff0fff, OP_0_B_LIMM, F_F },
  { "adds",   0x2ec67000, 0xffff7020, OP_0_LIMM_C, F_F | F_CC },
  { "addsdw", 0x28280000, 0xf8ff0000, OP_A_B_C, F_F },
  { "addsdw", 0x28680000, 0xf8ff0000, OP_A_B_U6, F_F },
  { "addsdw", 0x28a80000, 0xf8ff0000, OP_B_B_S12, F_F },
  { "addsdw", 0x28e80000, 0xf8ff0020, OP_B_B_C, F_F | F_CC },
  { "addsdw", 0x28e80020, 0xf8ff0020, OP_B_B_U6, F_F | F_CC },
  { "addsdw", 0x2e287000, 0xffff7000, OP_A_LIMM_C, F_F },
  { "addsdw", 0x28280f80, 0xf8ff0fc0, OP_A_B_LIMM, F_F },
  { "addsdw", 0x28e80f80, 0xf8ff0fc0, OP_B_B_LIMM, F_F | F_CC },
  { "addsdw", 0x2828003e, 0xf8ff003f, OP_0_B_C, F_F },
  { "addsdw", 0x2868003e, 0xf8ff003f, OP_0_B_U6, F_F },
  { "addsdw", 0x2ee87000, 0xffff7020, OP_0_LIMM_C, F_F | F_CC },
  { "and",    0x20040000, 0xf8ff0000, OP_A_B_C, F_F },
  { "and",    0x20440000, 0xf8ff0000, OP_A_B_U6, F_F },
  { "and",    0x20840000, 0xf8ff0000, OP_B_B_S12, F_F },
  { "and",    0x20c40000, 0xf8ff0020, OP_B_B_C, F_F | F_CC },
  { "and",    0x20c40020, 0xf8ff0020, OP_B_B_U6, F_F | F_CC },
  { "and",    0x26047000, 0xffff7000, OP_A_LIMM_C, F_F },
  { "and",    0x20040f80, 0xf8ff0fc0, OP_A_B_LIMM, F_F },
  { "and",    0x20c40f80, 0xf8ff0fe0, OP_B_B_LIMM, F_F | F_CC },
  { "and",    0x2004003e, 0xf8ff003f, OP_0_B_C, F_F },
  { "and",    0x2044003e, 0xf8ff003f, OP_0_B_U6, F_F },
  { "and",    0x20040fbe, 0xf8ff0fff, OP_0_B_LIMM, F_F },
  { "and",    0x26c47000, 0xffff7020, OP_0_LIMM_C, F_F | F_CC },
  { "asl",    0x202f0000, 0xf8ff003f, OP_B_C, F_F },
  { "asl",    0x206f0000, 0xf8ff003f, OP_B_U6, F_F },
  { "asl",    0x202f0f80, 0xf8ff0fff, OP_B_LIMM, F_F },
  { "asl",    0x262f7000, 0xffff703f, OP_0_C, F_F },
  { "asl",    0x266f7000, 0xffff703f, OP_0_U6, F_F },
  { "asl",    0x262f7f80, 0xffff7fff, OP_0_LIMM, F_F },
  { "asl",    0x28000000, 0xf8ff0000, OP_A_B_C, F_F },
  { "asl",    0x28400000, 0xf8ff0000, OP_A_B_U6, F_F },
  { "asl",    0x28800000, 0xf8ff0000, OP_B_B_S12, F_F },
  { "asl",    0x28c00000, 0xf8ff0020, OP_B_B_C, F_F | F_CC },
  { "asl",    0x28c00020, 0xf8ff0020, OP_B_B_U6, F_F | F_CC },
  { "asl",    0x2e007000, 0xffff7000, OP_A_LIMM_C, F_F },
  { "asl",    0x28000f80, 0xf8ff0fc0, OP_A_B_LIMM, F_F },
  { "asl",    0x28c00f80, 0xf8ff0fe0, OP_B_B_LIMM, F_F | F_CC },
  { "asl",    0x2800003e, 0xf8ff003f, OP_0_B_C, F_F },
  { "asl",    0x2840003e, 0xf8ff003f, OP_0_B_U6, F_F },
  { "asl",    0x2ec07000, 0xffff7020, OP_0_LIMM_C, F_F | F_CC },
  { "asls",   0x280a0000, 0xf8ff0000, OP_A_B_C, F_F },
  { "asls",   0x284a0000, 0xf8ff0000, OP_A_B_U6, F_F },
  { "asls",   0x288a0000, 0xf8ff0000, OP_B_B_S12, F_F },
  { "asls",   0x28ca0000, 0xf8ff0020, OP_B_B_C, F_F | F_CC },
  { "asls",   0x28ca0020, 0xf8ff0020, OP_B_B_U6, F_F | F_CC },
  { "asls",   0x2e0a7000, 0xffff7000, OP_A_LIMM_C, F_F },
  { "asls",   0x280a0f80, 0xf8ff0fc0, OP_A_B_LIMM, F_F },
  { "asls",   0x28ca0f80, 0xf8ff0fc0, OP_B_B_LIMM, F_F | F_CC },
  { "asls",   0x280a003e, 0xf8ff003f, OP_0_B_C, F_F },
  { "asls",   0x284a003e, 0xf8ff003f, OP_0_B_U6, F_F },
  { "asls",   0x2eca7000, 0xffff7020, OP_0_LIMM_C, F_F | F_CC },
  { "asr",    0x202f0001, 0xf8ff003f, OP_B_C, F_F },
  { "asr",    0x206f0001, 0xf8ff003f, OP_B_U6, F_F },
  { "asr",    0x202f0f81, 0xf8ff0fff, OP_B_LIMM, F_F },
  { "asr",    0x262f7001, 0xffff703f, OP_0_C, F_F },
  { "asr",    0x266f7001, 0xffff703f, OP_0_U6, F_F },
  { "asr",    0x262f7f81, 0xffff7fff, OP_0_LIMM, F_F },
  { "asr",    0x28020000, 0xf8ff0000, OP_A_B_C, F_F },
  { "asr",    0x28420000, 0xf8ff0000, OP_A_B_U6, F_F },
  { "asr",    0x28820000, 0xf8ff0000, OP_B_B_S12, F_F },
  { "asr",    0x28c20000, 0xf8ff0020, OP_B_B_C, F_F | F_CC },
  { "asr",    0x28c20020, 0xf8ff0020, OP_B_B_U6, F_F | F_CC },
  { "asr",    0x2e027000, 0xffff7000, OP_A_LIMM_C, F_F },
  { "asr",    0x28020f80, 0xf8ff0fc0, OP_A_B_LIMM, F_F },
  { "asr",    0x28c20f80, 0xf8ff0fe0, OP_B_B_LIMM, F_F | F_CC },
  { "asr",    0x2802003e, 0xf8ff003f, OP_0_B_C, F_F },
  { "asr",    0x2842003e, 0xf8ff003f, OP_0_B_U6, F_F },
  { "asr",    0x2ec27000, 0xffff7020, OP_0_LIMM_C, F_F | F_CC },
  { "asrs",   0x280b0000, 0xf8ff0000, OP_A_B_C, F_F },
  { "asrs",   0x284b0000, 0xf8ff0000, OP_A_B_U6, F_F },
  { "asrs",   0x288b0000, 0xf8ff0000, OP_B_B_S12, F_F },
  { "asrs",   0x28cb0000, 0xf8ff0020, OP_B_B_C, F_F | F_CC },
  { "asrs",   0x28cb0020, 0xf8ff0020, OP_B_B_U6, F_F | F_CC },
  { "asrs",   0x2e0b7000, 0xffff7000, OP_A_LIMM_C, F_F },
  { "asrs",   0x280b0f80, 0xf8ff0fc0, OP_A_B_LIMM, F_F },
  { "asrs",   0x28cb0f80, 0xf8ff0fc0, OP_B_B_LIMM, F_F | F_CC },
  { "asrs",   0x280b003e, 0xf8ff003f, OP_0_B_C, F_F },
  { "asrs",   0x284b003e, 0xf8ff003f, OP_0_B_U6, F_F },
  { "asrs",   0x2ecb7000, 0xffff7020, OP_0_LIMM_C, F_F | F_CC },
  { "bbit0",  0x0801000e, 0xf801001f, OP_B_C_O9, F_D },
  { "bbit0",  0x0801001e, 0xf801001f, OP_B_U6_O9, F_D },
  { "bbit1",  0x0801000f, 0xf801001f, OP_B_C_O9, F_D },
  { "bbit1",  0x0801001f, 0xf801001f, OP_B_U6_O9, F_D },
  { NULL, 0, 0, 0, 0 },
};

struct _table_arc16 table_arc16[] =
{
  { "abs_s",  0x7811, 0xf81f, OP_B_C },
  { "add_s",  0x6018, 0xf818, OP_A_B_C },
  { "add_s",  0x6800, 0xf818, OP_C_B_U3 },
  { "add_s",  0x70c7, 0xf8ff, OP_B_B_LIMM },
  { "add_s",  0x7000, 0xf818, OP_B_B_H },
  { "add_s",  0xc080, 0xf8e0, OP_B_SP_U7 },
  { "add_s",  0xc0a0, 0xffe0, OP_SP_SP_U7 },
  { "add_s",  0xce00, 0xfe00, OP_R0_GP_S9 },
  { "add_s",  0xe000, 0xf880, OP_B_B_U7 },
  { "add1_s", 0x7814, 0xf81f, OP_B_B_C },
  { "add2_s", 0x7815, 0xf81f, OP_B_B_C },
  { "add3_s", 0x7816, 0xf81f, OP_B_B_C },
  { "and_s",  0x7804, 0xf81f, OP_B_B_C },
  { "asl_s",  0x781b, 0xf81f, OP_B_C },
  { "asl_s",  0x6810, 0xf818, OP_C_B_U3 },
  { "asl_s",  0x7818, 0xf81f, OP_B_B_C },
  { "asl_s",  0xb800, 0xf8e0, OP_B_B_U5 },
  { "asr_s",  0x781c, 0xf81f, OP_B_C },
  { "asr_s",  0x6818, 0xf818, OP_C_B_U3 },
  { "asr_s",  0x781a, 0xf81f, OP_B_B_C },
  { "asr_s",  0xb840, 0xf8e0, OP_B_B_U5 },
  { NULL, 0, 0, 0 },
};

