/**
 *  naken_asm assembler.
 *  Author: Michael Kohn
 *   Email: mike@mikekohn.net
 *     Web: http://www.mikekohn.net/
 * License: GPLv3
 *
 * Copyright 2010-2020 by Michael Kohn
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "table/m8c.h"

struct _table_m8c table_m8c[] =
{
  { "adc",  0x09, 0xff, OP_A, OP_EXPR, 2, 4 },
  { "adc",  0x0a, 0xff, OP_A, OP_INDEX_EXPR, 2, 6 },
  { "adc",  0x0b, 0xff, OP_A, OP_INDEX_X_EXPR, 2, 7 },
  { "adc",  0x0c, 0xff, OP_INDEX_EXPR, OP_A, 2, 7 },
  { "adc",  0x0d, 0xff, OP_INDEX_X_EXPR, OP_A, 2, 8 },
  { "adc",  0x0e, 0xff, OP_INDEX_EXPR, OP_EXPR, 3, 9 },
  { "adc",  0x0f, 0xff, OP_INDEX_X_EXPR, OP_EXPR, 3, 10 },
  { "add",  0x01, 0xff, OP_A, OP_EXPR, 2, 4 },
  { "add",  0x02, 0xff, OP_A, OP_INDEX_EXPR, 2, 6 },
  { "add",  0x03, 0xff, OP_A, OP_INDEX_X_EXPR, 2, 7 },
  { "add",  0x04, 0xff, OP_INDEX_EXPR, OP_A, 2, 7 },
  { "add",  0x05, 0xff, OP_INDEX_X_EXPR, OP_A, 2, 8 },
  { "add",  0x06, 0xff, OP_INDEX_EXPR, OP_EXPR, 3, 9 },
  { "add",  0x07, 0xff, OP_INDEX_X_EXPR, OP_EXPR, 3, 10 },
  { "add",  0x38, 0xff, OP_SP, OP_EXPR, 2, 5 },
  { "and",  0x21, 0xff, OP_A, OP_EXPR, 2, 4 },
  { "and",  0x22, 0xff, OP_A, OP_INDEX_EXPR, 2, 6 },
  { "and",  0x23, 0xff, OP_A, OP_INDEX_X_EXPR, 2, 7 },
  { "and",  0x24, 0xff, OP_INDEX_EXPR, OP_A, 2, 7 },
  { "and",  0x25, 0xff, OP_INDEX_X_EXPR, OP_A, 2, 8 },
  { "and",  0x26, 0xff, OP_INDEX_EXPR, OP_EXPR, 3, 9 },
  { "and",  0x27, 0xff, OP_INDEX_X_EXPR, OP_EXPR, 3, 10 },
  { "and",  0x41, 0xff, OP_REG_INDEX_EXPR, OP_EXPR, 3, 9 },
  { "and",  0x42, 0xff, OP_REG_INDEX_X_EXPR, OP_EXPR, 3, 10 },
  { "and",  0x70, 0xff, OP_F, OP_EXPR, 2, 4 },
  { "asl",  0x64, 0xff, OP_A, OP_NONE, 1, 4 },
  { "asl",  0x65, 0xff, OP_INDEX_EXPR, OP_NONE, 2, 7 },
  { "asl",  0x66, 0xff, OP_INDEX_X_EXPR, OP_NONE, 2, 8 },
  { "asr",  0x67, 0xff, OP_A, OP_NONE, 1, 4 },
  { "asr",  0x68, 0xff, OP_INDEX_EXPR, OP_NONE, 2, 7 },
  { "asr",  0x69, 0xff, OP_INDEX_X_EXPR, OP_NONE, 2, 8 },
  { "call", 0x90, 0xf0, OP_EXPR_S12, OP_NONE, 2, 11 },
  { "cmp",  0x39, 0xff, OP_A, OP_EXPR, 2, 5 },
  { "cmp",  0x3a, 0xff, OP_A, OP_INDEX_EXPR, 2, 7 },
  { "cmp",  0x3b, 0xff, OP_A, OP_INDEX_X_EXPR, 2, 8 },
  { "cmp",  0x3c, 0xff, OP_INDEX_EXPR, OP_EXPR, 3, 8 },
  { "cmp",  0x3d, 0xff, OP_INDEX_X_EXPR, OP_EXPR, 3, 9 },
  { "cpl",  0x73, 0xff, OP_A, OP_NONE, 1, 4 },
  { "dec",  0x78, 0xff, OP_A, OP_NONE, 1, 4 },
  { "dec",  0x79, 0xff, OP_X, OP_NONE, 1, 4 },
  { "dec",  0x7a, 0xff, OP_INDEX_EXPR, OP_NONE, 2, 7 },
  { "dec",  0x7b, 0xff, OP_INDEX_X_EXPR, OP_NONE, 2, 8 },
  { "halt", 0x30, 0xff, OP_NONE, OP_NONE, 1, 9 },
  { "inc",  0x74, 0xff, OP_A, OP_NONE, 1, 4 },
  { "inc",  0x75, 0xff, OP_X, OP_NONE, 1, 4 },
  { "inc",  0x76, 0xff, OP_INDEX_EXPR, OP_NONE, 2, 7 },
  { "inc",  0x77, 0xff, OP_INDEX_X_EXPR, OP_NONE, 2, 8 },
  { "index",0xf0, 0xf0, OP_EXPR_S12, OP_NONE, 2, 13 },
  { "jacc", 0xe0, 0xf0, OP_EXPR_S12_JUMP, OP_NONE, 2, 7 },
  { "jc",   0xc0, 0xf0, OP_EXPR_S12_JUMP, OP_NONE, 2, 5 },
  { "jmp",  0x80, 0xf0, OP_EXPR_S12_JUMP, OP_NONE, 2, 5 },
  { "jnc",  0xd0, 0xf0, OP_EXPR_S12_JUMP, OP_NONE, 2, 5 },
  { "jnz",  0xb0, 0xf0, OP_EXPR_S12_JUMP, OP_NONE, 2, 5 },
  { "jz",   0xa0, 0xf0, OP_EXPR_S12_JUMP, OP_NONE, 2, 5 },
  { "lcall",0x7c, 0xff, OP_EXPR_U16, OP_NONE, 3, 13 },
  { "ljmp", 0x7d, 0xff, OP_EXPR_U16, OP_NONE, 3, 7 },
  { "mov",  0x4f, 0xff, OP_X, OP_SP, 1, 4 },
  { "mov",  0x50, 0xff, OP_A, OP_EXPR, 2, 4 },
  { "mov",  0x51, 0xff, OP_A, OP_INDEX_EXPR, 2, 5 },
  { "mov",  0x52, 0xff, OP_A, OP_INDEX_X_EXPR, 2, 6 },
  { "mov",  0x53, 0xff, OP_INDEX_EXPR, OP_A, 2, 5 },
  { "mov",  0x54, 0xff, OP_INDEX_X_EXPR, OP_A, 2, 6 },
  { "mov",  0x55, 0xff, OP_INDEX_EXPR, OP_EXPR, 3, 8 },
  { "mov",  0x56, 0xff, OP_INDEX_X_EXPR, OP_EXPR, 3, 9 },
  { "mov",  0x57, 0xff, OP_X, OP_EXPR, 2, 4 },
  { "mov",  0x58, 0xff, OP_X, OP_INDEX_EXPR, 2, 6 },
  { "mov",  0x59, 0xff, OP_X, OP_INDEX_X_EXPR, 2, 7 },
  { "mov",  0x5a, 0xff, OP_INDEX_EXPR, OP_X, 2, 5 },
  { "mov",  0x5b, 0xff, OP_A, OP_X, 1, 4 },
  { "mov",  0x5c, 0xff, OP_X, OP_A, 1, 4 },
  { "mov",  0x5d, 0xff, OP_A, OP_REG_INDEX_EXPR, 2, 6 },
  { "mov",  0x5e, 0xff, OP_A, OP_REG_INDEX_X_EXPR, 2, 7 },
  { "mov",  0x5f, 0xff, OP_INDEX_EXPR, OP_INDEX_EXPR, 3, 10 },
  { "mov",  0x60, 0xff, OP_REG_INDEX_EXPR, OP_A, 2, 5 },
  { "mov",  0x61, 0xff, OP_REG_INDEX_X_EXPR, OP_A, 2, 6 },
  { "mov",  0x62, 0xff, OP_REG_INDEX_EXPR, OP_EXPR, 3, 8 },
  { "mov",  0x63, 0xff, OP_REG_INDEX_X_EXPR, OP_EXPR, 3, 9 },
  { "mvi",  0x3e, 0xff, OP_A, OP_INDEX_EXPR, 2, 10 },
  { "mvi",  0x3f, 0xff, OP_INDEX_EXPR, OP_A, 2, 10 },
  { "nop",  0x40, 0xff, OP_NONE, OP_NONE, 1, 4 },
  { "or",   0x29, 0xff, OP_A, OP_EXPR, 2, 4 },
  { "or",   0x2a, 0xff, OP_A, OP_INDEX_EXPR, 2, 6 },
  { "or",   0x2b, 0xff, OP_A, OP_INDEX_X_EXPR, 2, 7 },
  { "or",   0x2c, 0xff, OP_INDEX_EXPR, OP_A, 2, 7 },
  { "or",   0x2d, 0xff, OP_INDEX_X_EXPR, OP_A, 2, 8 },
  { "or",   0x2e, 0xff, OP_INDEX_EXPR, OP_EXPR, 3, 9 },
  { "or",   0x2f, 0xff, OP_INDEX_X_EXPR, OP_EXPR, 3, 10 },
  { "or",   0x43, 0xff, OP_REG_INDEX_EXPR, OP_EXPR, 3, 9 },
  { "or",   0x44, 0xff, OP_REG_INDEX_X_EXPR, OP_EXPR, 3, 10 },
  { "or",   0x71, 0xff, OP_F, OP_EXPR, 2, 4 },
  { "pop",  0x18, 0xff, OP_A, OP_NONE, 1, 5 },
  { "pop",  0x20, 0xff, OP_X, OP_NONE, 1, 5 },
  { "push", 0x08, 0xff, OP_A, OP_NONE, 1, 4 },
  { "push", 0x10, 0xff, OP_X, OP_NONE, 1, 4 },
  { "ret",  0x7f, 0xff, OP_NONE, OP_NONE, 1, 8 },
  { "reti", 0x7e, 0xff, OP_NONE, OP_NONE, 1, 10 },
  { "rlc",  0x6a, 0xff, OP_A, OP_NONE, 1, 4 },
  { "rlc",  0x6b, 0xff, OP_INDEX_EXPR, OP_NONE, 2, 7 },
  { "rlc",  0x6c, 0xff, OP_INDEX_X_EXPR, OP_NONE, 2, 8 },
  { "romx", 0x28, 0xff, OP_NONE, OP_NONE, 1, 11 },
  { "rrc",  0x6d, 0xff, OP_A, OP_NONE, 1, 4 },
  { "rrc",  0x6e, 0xff, OP_INDEX_EXPR, OP_NONE, 2, 7 },
  { "rrc",  0x6f, 0xff, OP_INDEX_X_EXPR, OP_NONE, 2, 8 },
  { "sbb",  0x19, 0xff, OP_A, OP_EXPR, 2, 4 },
  { "sbb",  0x1a, 0xff, OP_A, OP_INDEX_EXPR, 2, 6 },
  { "sbb",  0x1b, 0xff, OP_A, OP_INDEX_X_EXPR, 2, 7 },
  { "sbb",  0x1c, 0xff, OP_INDEX_EXPR, OP_A, 2, 7 },
  { "sbb",  0x1d, 0xff, OP_INDEX_X_EXPR, OP_A, 2, 8 },
  { "sbb",  0x1e, 0xff, OP_INDEX_EXPR, OP_EXPR, 3, 9 },
  { "sbb",  0x1f, 0xff, OP_INDEX_X_EXPR, OP_EXPR, 3, 10 },
  { "sub",  0x11, 0xff, OP_A, OP_EXPR, 2, 4 },
  { "sub",  0x12, 0xff, OP_A, OP_INDEX_EXPR, 2, 6 },
  { "sub",  0x13, 0xff, OP_A, OP_INDEX_X_EXPR, 2, 7 },
  { "sub",  0x14, 0xff, OP_INDEX_EXPR, OP_A, 2, 7 },
  { "sub",  0x15, 0xff, OP_INDEX_X_EXPR, OP_A, 2, 8 },
  { "sub",  0x16, 0xff, OP_INDEX_EXPR, OP_EXPR, 3, 9 },
  { "sub",  0x17, 0xff, OP_INDEX_X_EXPR, OP_EXPR, 3, 10 },
  { "swap", 0x4b, 0xff, OP_A, OP_X, 1, 5 },
  { "swap", 0x4c, 0xff, OP_A, OP_INDEX_EXPR, 2, 7 },
  { "swap", 0x4d, 0xff, OP_X, OP_INDEX_EXPR, 2, 7 },
  { "swap", 0x4e, 0xff, OP_A, OP_SP, 1, 5 },
  { "ssc",  0x00, 0xff, OP_NONE, OP_NONE, 1, 15 },
  { "tst",  0x47, 0xff, OP_INDEX_EXPR, OP_EXPR, 3, 8 },
  { "tst",  0x48, 0xff, OP_INDEX_X_EXPR, OP_EXPR, 3, 9 },
  { "tst",  0x49, 0xff, OP_REG_INDEX_EXPR, OP_EXPR, 3, 9 },
  { "tst",  0x4a, 0xff, OP_REG_INDEX_X_EXPR, OP_EXPR, 3, 10 },
  { "xor",  0x31, 0xff, OP_A, OP_EXPR, 2, 4 },
  { "xor",  0x32, 0xff, OP_A, OP_INDEX_EXPR, 2, 6 },
  { "xor",  0x33, 0xff, OP_A, OP_INDEX_X_EXPR, 2, 7 },
  { "xor",  0x34, 0xff, OP_INDEX_EXPR, OP_A, 2, 7 },
  { "xor",  0x35, 0xff, OP_INDEX_X_EXPR, OP_A, 2, 8 },
  { "xor",  0x36, 0xff, OP_INDEX_EXPR, OP_EXPR, 3, 9 },
  { "xor",  0x37, 0xff, OP_INDEX_X_EXPR, OP_EXPR, 3, 10 },
  { "xor",  0x45, 0xff, OP_REG_INDEX_EXPR, OP_EXPR, 3, 9 },
  { "xor",  0x46, 0xff, OP_REG_INDEX_X_EXPR, OP_EXPR, 3, 10 },
  { "xor",  0x72, 0xff, OP_F, OP_EXPR, 2, 4 },
  { NULL,   0x00, 0x00, OP_NONE, 0, 0 }
};

