/**
 *  naken_asm assembler.
 *  Author: Michael Kohn
 *   Email: mike@mikekohn.net
 *     Web: http://www.mikekohn.net/
 * License: GPLv3
 *
 * Copyright 2010-2019 by Michael Kohn
 *
 */

#include <stdlib.h>

#include "table/webasm.h"

struct _table_webasm table_webasm[] =
{
  { "unreachable",    0x00, WEBASM_OP_NONE },
  { "nop",            0x01, WEBASM_OP_NONE },
  { "block",          0x02, WEBASM_OP_BLOCK_TYPE },
  { "loop",           0x03, WEBASM_OP_BLOCK_TYPE },
  { "if",             0x04, WEBASM_OP_BLOCK_TYPE },
  { "else",           0x05, WEBASM_OP_NONE },
  { "end",            0x0b, WEBASM_OP_NONE },
  { "br",             0x0c, WEBASM_OP_RELATIVE_DEPTH },
  { "br_if",          0x0d, WEBASM_OP_RELATIVE_DEPTH },
  { "br_table",       0x0e, WEBASM_OP_TABLE },
  { "return",         0x0f, WEBASM_OP_NONE },
  { "call",           0x10, WEBASM_OP_FUNCTION_INDEX },
  { "call_indirect",  0x11, WEBASM_OP_INDIRECT },
  { "drop",           0x1a, WEBASM_OP_NONE },
  { "select",         0x1b, WEBASM_OP_NONE },
  { "get_local",      0x20, WEBASM_OP_LOCAL_INDEX },
  { "set_local",      0x21, WEBASM_OP_LOCAL_INDEX },
  { "tee_local",      0x22, WEBASM_OP_LOCAL_INDEX },
  { "get_global",     0x23, WEBASM_OP_GLOBAL_INDEX },
  { "set_global",     0x24, WEBASM_OP_GLOBAL_INDEX },
  { "i32.load",       0x28, WEBASM_OP_MEMORY_IMMEDIATE },
  { "i64.load",       0x29, WEBASM_OP_MEMORY_IMMEDIATE },
  { "f32.load",       0x2a, WEBASM_OP_MEMORY_IMMEDIATE },
  { "f64.load",       0x2b, WEBASM_OP_MEMORY_IMMEDIATE },
  { "i32.load8_s",    0x2c, WEBASM_OP_MEMORY_IMMEDIATE },
  { "i32.load8_u",    0x2d, WEBASM_OP_MEMORY_IMMEDIATE },
  { "i32.load16_s",   0x2e, WEBASM_OP_MEMORY_IMMEDIATE },
  { "i32.load16_u",   0x2f, WEBASM_OP_MEMORY_IMMEDIATE },
  { "i64.load8_s",    0x30, WEBASM_OP_MEMORY_IMMEDIATE },
  { "i64.load8_u",    0x31, WEBASM_OP_MEMORY_IMMEDIATE },
  { "i64.load16_s",   0x32, WEBASM_OP_MEMORY_IMMEDIATE },
  { "i64.load16_u",   0x33, WEBASM_OP_MEMORY_IMMEDIATE },
  { "i64.load32_s",   0x34, WEBASM_OP_MEMORY_IMMEDIATE },
  { "i64.load32_u",   0x35, WEBASM_OP_MEMORY_IMMEDIATE },
  { "i32.store",      0x36, WEBASM_OP_MEMORY_IMMEDIATE },
  { "i64.store",      0x37, WEBASM_OP_MEMORY_IMMEDIATE },
  { "f32.store",      0x38, WEBASM_OP_MEMORY_IMMEDIATE },
  { "f64.store",      0x39, WEBASM_OP_MEMORY_IMMEDIATE },
  { "i32.store8",     0x3a, WEBASM_OP_MEMORY_IMMEDIATE },
  { "i32.store16",    0x3b, WEBASM_OP_MEMORY_IMMEDIATE },
  { "i64.store8",     0x3c, WEBASM_OP_MEMORY_IMMEDIATE },
  { "i64.store16",    0x3d, WEBASM_OP_MEMORY_IMMEDIATE },
  { "i64.store32",    0x3e, WEBASM_OP_MEMORY_IMMEDIATE },
  { "current_memory", 0x3f, WEBASM_OP_ILLEGAL },
  { "grow_memory",    0x40, WEBASM_OP_ILLEGAL },
  { "i32.const",      0x41, WEBASM_OP_VARINT32 },
  { "i64.const",      0x42, WEBASM_OP_VARINT64 },
  { "f32.const",      0x43, WEBASM_OP_UINT32 },
  { "f64.const",      0x44, WEBASM_OP_UINT64 },
  { "i32.eqz",        0x45, WEBASM_OP_NONE },
  { "i32.eq",         0x46, WEBASM_OP_NONE },
  { "i32.ne",         0x47, WEBASM_OP_NONE },
  { "i32.lt_s",       0x48, WEBASM_OP_NONE },
  { "i32.lt_u",       0x49, WEBASM_OP_NONE },
  { "i32.gt_s",       0x4a, WEBASM_OP_NONE },
  { "i32.gt_u",       0x4b, WEBASM_OP_NONE },
  { "i32.le_s",       0x4c, WEBASM_OP_NONE },
  { "i32.le_u",       0x4d, WEBASM_OP_NONE },
  { "i32.ge_s",       0x4e, WEBASM_OP_NONE },
  { "i32.ge_u",       0x4f, WEBASM_OP_NONE },
  { "i64.eqz",        0x50, WEBASM_OP_NONE },
  { "i64.eq",         0x51, WEBASM_OP_NONE },
  { "i64.ne",         0x52, WEBASM_OP_NONE },
  { "i64.lt_s",       0x53, WEBASM_OP_NONE },
  { "i64.lt_u",       0x54, WEBASM_OP_NONE },
  { "i64.gt_s",       0x55, WEBASM_OP_NONE },
  { "i64.gt_u",       0x56, WEBASM_OP_NONE },
  { "i64.le_s",       0x57, WEBASM_OP_NONE },
  { "i64.le_u",       0x58, WEBASM_OP_NONE },
  { "i64.ge_s",       0x59, WEBASM_OP_NONE },
  { "i64.ge_u",       0x5a, WEBASM_OP_NONE },
  { "f32.eq",         0x5b, WEBASM_OP_NONE },
  { "f32.ne",         0x5c, WEBASM_OP_NONE },
  { "f32.lt",         0x5d, WEBASM_OP_NONE },
  { "f32.gt",         0x5e, WEBASM_OP_NONE },
  { "f32.le",         0x5f, WEBASM_OP_NONE },
  { "f32.ge",         0x60, WEBASM_OP_NONE },
  { "f64.eq",         0x61, WEBASM_OP_NONE },
  { "f64.ne",         0x62, WEBASM_OP_NONE },
  { "f64.lt",         0x63, WEBASM_OP_NONE },
  { "f64.gt",         0x64, WEBASM_OP_NONE },
  { "f64.le",         0x65, WEBASM_OP_NONE },
  { "f64.ge",         0x66, WEBASM_OP_NONE },
  { "i32.clz",        0x67, WEBASM_OP_NONE },
  { "i32.ctz",        0x68, WEBASM_OP_NONE },
  { "i32.popcnt",     0x69, WEBASM_OP_NONE },
  { "i32.add",        0x6a, WEBASM_OP_NONE },
  { "i32.sub",        0x6b, WEBASM_OP_NONE },
  { "i32.mul",        0x6c, WEBASM_OP_NONE },
  { "i32.div_s",      0x6d, WEBASM_OP_NONE },
  { "i32.div_u",      0x6e, WEBASM_OP_NONE },
  { "i32.rem_s",      0x6f, WEBASM_OP_NONE },
  { "i32.rem_u",      0x70, WEBASM_OP_NONE },
  { "i32.and",        0x71, WEBASM_OP_NONE },
  { "i32.or",         0x72, WEBASM_OP_NONE },
  { "i32.xor",        0x73, WEBASM_OP_NONE },
  { "i32.shl",        0x74, WEBASM_OP_NONE },
  { "i32.shr_s",      0x75, WEBASM_OP_NONE },
  { "i32.shr_u",      0x76, WEBASM_OP_NONE },
  { "i32.rotl",       0x77, WEBASM_OP_NONE },
  { "i32.rotr",       0x78, WEBASM_OP_NONE },
  { "i64.clz",        0x79, WEBASM_OP_NONE },
  { "i64.ctz",        0x7a, WEBASM_OP_NONE },
  { "i64.popcnt",     0x7b, WEBASM_OP_NONE },
  { "i64.add",        0x7c, WEBASM_OP_NONE },
  { "i64.sub",        0x7d, WEBASM_OP_NONE },
  { "i64.mul",        0x7e, WEBASM_OP_NONE },
  { "i64.div_s",      0x7f, WEBASM_OP_NONE },
  { "i64.div_u",      0x80, WEBASM_OP_NONE },
  { "i64.rem_s",      0x81, WEBASM_OP_NONE },
  { "i64.rem_u",      0x82, WEBASM_OP_NONE },
  { "i64.and",        0x83, WEBASM_OP_NONE },
  { "i64.or",         0x84, WEBASM_OP_NONE },
  { "i64.xor",        0x85, WEBASM_OP_NONE },
  { "i64.shl",        0x86, WEBASM_OP_NONE },
  { "i64.shr_s",      0x87, WEBASM_OP_NONE },
  { "i64.shr_u",      0x88, WEBASM_OP_NONE },
  { "i64.rotl",       0x89, WEBASM_OP_NONE },
  { "i64.rotr",       0x8a, WEBASM_OP_NONE },
  { "f32.abs",        0x8b, WEBASM_OP_NONE },
  { "f32.neg",        0x8c, WEBASM_OP_NONE },
  { "f32.ceil",       0x8d, WEBASM_OP_NONE },
  { "f32.floor",      0x8e, WEBASM_OP_NONE },
  { "f32.trunc",      0x8f, WEBASM_OP_NONE },
  { "f32.nearest",    0x90, WEBASM_OP_NONE },
  { "f32.sqrt",       0x91, WEBASM_OP_NONE },
  { "f32.add",        0x92, WEBASM_OP_NONE },
  { "f32.sub",        0x93, WEBASM_OP_NONE },
  { "f32.mul",        0x94, WEBASM_OP_NONE },
  { "f32.div",        0x95, WEBASM_OP_NONE },
  { "f32.min",        0x96, WEBASM_OP_NONE },
  { "f32.max",        0x97, WEBASM_OP_NONE },
  { "f32.copysign",   0x98, WEBASM_OP_NONE },
  { "f64.abs",        0x99, WEBASM_OP_NONE },
  { "f64.neg",        0x9a, WEBASM_OP_NONE },
  { "f64.ceil",       0x9b, WEBASM_OP_NONE },
  { "f64.floor",      0x9c, WEBASM_OP_NONE },
  { "f64.trunc",      0x9d, WEBASM_OP_NONE },
  { "f64.nearest",    0x9e, WEBASM_OP_NONE },
  { "f64.sqrt",       0x9f, WEBASM_OP_NONE },
  { "f64.add",        0xa0, WEBASM_OP_NONE },
  { "f64.sub",        0xa1, WEBASM_OP_NONE },
  { "f64.mul",        0xa2, WEBASM_OP_NONE },
  { "f64.div",        0xa3, WEBASM_OP_NONE },
  { "f64.min",        0xa4, WEBASM_OP_NONE },
  { "f64.max",        0xa5, WEBASM_OP_NONE },
  { "f64.copysign",   0xa6, WEBASM_OP_NONE },
  { "i32.wrap/i64",        0xa7, WEBASM_OP_NONE },
  { "i32.trunc_s/f32",     0xa8, WEBASM_OP_NONE },
  { "i32.trunc_u/f32",     0xa9, WEBASM_OP_NONE },
  { "i32.trunc_s/f64",     0xaa, WEBASM_OP_NONE },
  { "i32.trunc_u/f64",     0xab, WEBASM_OP_NONE },
  { "i64.extend_s/i32",    0xac, WEBASM_OP_NONE },
  { "i64.extend_u/i32",    0xad, WEBASM_OP_NONE },
  { "i64.trunc_s/f32",     0xae, WEBASM_OP_NONE },
  { "i64.trunc_u/f32",     0xaf, WEBASM_OP_NONE },
  { "i64.trunc_s/f64",     0xb0, WEBASM_OP_NONE },
  { "i64.trunc_u/f64",     0xb1, WEBASM_OP_NONE },
  { "f32.convert_s/i32",   0xb2, WEBASM_OP_NONE },
  { "f32.convert_u/i32",   0xb3, WEBASM_OP_NONE },
  { "f32.convert_s/i64",   0xb4, WEBASM_OP_NONE },
  { "f32.convert_u/i64",   0xb5, WEBASM_OP_NONE },
  { "f32.demote/f64",      0xb6, WEBASM_OP_NONE },
  { "f64.convert_s/i32",   0xb7, WEBASM_OP_NONE },
  { "f64.convert_u/i32",   0xb8, WEBASM_OP_NONE },
  { "f64.convert_s/i64",   0xb9, WEBASM_OP_NONE },
  { "f64.convert_u/i64",   0xba, WEBASM_OP_NONE },
  { "f64.promote/f32",     0xbb, WEBASM_OP_NONE },
  { "i32.reinterpret/f32", 0xbc, WEBASM_OP_NONE },
  { "i64.reinterpret/f64", 0xbd, WEBASM_OP_NONE },
  { "f32.reinterpret/i32", 0xbe, WEBASM_OP_NONE },
  { "f64.reinterpret/i64", 0xbf, WEBASM_OP_NONE },
  { NULL,                  0x00, WEBASM_OP_ILLEGAL },
};

struct _webasm_types webasm_types[] =
{
  { "i32",     0x7f },
  { "i64",     0x7e },
  { "f32",     0x7d },
  { "f64",     0x7c },
  { "anyfunc", 0x70 },
  { "func",    0x60 },
  { "void",    0x40 },
  { NULL,      0x00 },
};

